@@ -128,18 +128,6 @@ $CODE_MANIP{ 'save_to_lvar' } = sub {
     my $op = $self->ops->[ $self->current_line + 1 ];
 
     unless ( $op and $op->[0] =~ /^(?:print|and|or|dand|dor|push)/ ) { # ...
-
-        my $i = $self->current_line + 1;
-
-        while ( my $op = $self->ops->[ ++$i ] ) {
-            if ( $op->[0] =~ /^(?:print|and|or|dand|dor|push|for|macrocall)/ ) {
-                last;
-            }
-            if ( $op->[0] eq 'load_lvar_to_sb' ) {
-                return;
-            }
-        }
-
         $self->write_lines( sprintf( '%s;', $v )  );
     }
 
@@ -743,22 +731,27 @@ sub _check_logic {
 
         my $has_else_block = ($else_block_end >= $else_block_start);
 
-        my $last_opname = $ops->[ $i + $addr + $oparg - 1 ]->[0]; # check print or move_to_sb
-
         my $st_1st = $self->_spawn_child->_convert_opcode(
             [ @{ $ops }[ $if_block_start .. $if_block_end ] ]
         );
 
+        my $last_op = $ops->[ $i + $addr + $oparg - 1 ]; # check for ternary
+
         # treat as ternary
-        if ( $has_else_block and $last_opname and ( $last_opname eq 'print' or $last_opname eq 'move_to_sb' ) ) {
+        if ( $has_else_block and $last_op and $last_op->[0] =~ /^(?:print|move_to_sb|push|d?and|d?or)$/ ) {
 
             for (  $else_block_start .. $else_block_end ) { # 2
                 $self->stash->{ proc }->{ $_ }->{ skip } = 1; # skip
             }
 
-            my $st_2nd = $self->_spawn_child->_convert_opcode( # add $last_code for nested ternary
-                [ @{ $ops }[ $else_block_start .. $else_block_end ], [ $last_opname ] ]
-            );
+            # add $last_code for nested ternary
+            my $nested_ops = [ @{ $ops }[ $else_block_start .. $else_block_end ], $last_op ];
+            # when last op is 'push', must add pushmark for avoiding to access non-creatable array value
+            unshift @{ $nested_ops }, [ 'pushmark' ] if $last_op->[0] eq 'push';
+            # last op is 'd?and' or 'd?or', that op must be removed for properly assign
+            pop @{ $nested_ops } if $last_op->[0] =~ /^(?:d?and|d?or)$/;
+
+            my $st_2nd = $self->_spawn_child->_convert_opcode( $nested_ops );
 
         $self->sa( sprintf(  <<'CODE', $self->sa, _rm_tailed_lf( $st_1st->sa ), _rm_tailed_lf( $st_2nd->sa ) ) );
 cond_ternary( %s, sub { %s; }, sub { %s; } )
