@@ -5,7 +5,8 @@ use warnings;
 
 use parent qw(Exporter);
 our @EXPORT_OK = qw(
-    literal_to_value import_from
+    literal_to_value value_to_literal
+    import_from
     is_int any_in
     p
     $STRING $NUMBER $DEBUG
@@ -34,8 +35,11 @@ defined($DEBUG) or $DEBUG = $ENV{XSLATE} || '';
 
 sub is_int {
     my($s) = @_;
-    no warnings;
-    return $s eq int($s);
+    my $i = do {
+        no warnings;
+        int($s);
+    };
+    return $s eq $i && $i !~ /[^-0-9]/;
 }
 
 sub any_in {
@@ -56,7 +60,7 @@ my %esc2char = (
 
 sub literal_to_value {
     my($value) = @_;
-    return undef if not defined $value;
+    return $value if not defined $value;
 
     if($value =~ s/"(.*)"/$1/xms){
         $value =~ s/\\(.)/$esc2char{$1} || $1/xmseg;
@@ -77,11 +81,31 @@ sub literal_to_value {
     return $value;
 }
 
+my %char2esc = (
+    "\\" => '\\\\',
+    "\n" => '\\n',
+    "\r" => '\\r',
+    '"'  => '\\"',
+    '$'  => '\\$',
+    '@'  => '\\@',
+);
+my $value_chars = join '|', map { quotemeta } keys %char2esc;
+
+sub value_to_literal {
+    my($value) = @_;
+    return 'undef' if not defined $value;
+
+    if(is_int($value)){
+        return $value;
+    }
+    else {
+        $value =~ s/($value_chars)/$char2esc{$1}/xmsgeo;
+        return qq{"$value"};
+    }
+}
+
 sub import_from {
-    my $code = <<'T';
-package
-    Text::Xslate::Util::_import;
-T
+    my $code = '';
 
     local $Text::Xslate::Util::{'_import::'};
 
@@ -95,7 +119,9 @@ T
 
     my $e = do {
         local $@;
-        eval $code;
+        eval  qq{package}
+            . qq{ Text::Xslate::Util::_import;\n}
+            . $code;
         $@;
     };
     Carp::confess("Xslate: Failed to import:\n" . $code . $e) if $e;
